------------------------------------------------------------------------------
--
-- bivariate resultant computation
-- see "resultant.maple" for Maple interface functions
--
-- (c) 2000, Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
--
-- $Id: resultant.hs,v 1.1.1.1 2004/06/21 07:38:26 rmartine Exp $
--
-- 2004-06-01: Some minor changes to adapt to ghc-maple-0.2
-- Rafa.
------------------------------------------------------------------------------

import MapleAPI
import List
import Strategies
import Time

------------------------------------------------------------------------------
-- r(u) = resultant(p(u,v), q(u,v), u, v)
------------------------------------------------------------------------------
resultant :: MaplePoly -> MaplePoly -> MapleVar -> MapleVar -> MaplePoly
resultant p q u v =
  let 
    (a, b, cb, db) = preprocess p q u v
    (modules, product) = modlist a b cb db
    residues = map (\m -> mresultant m v a b db) modules
    coeffs = map (\r -> cra r modules) (transpose residues)
  in
    polynomial coeffs v product
    -- `using` \ _ -> parList (\ x -> rnf x) residues

------------------------------------------------------------------------------
--
-- Maple interface functions
--
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- (coeffP, coeffQ, coeffBound, degBound) = bounds p q u v
------------------------------------------------------------------------------
preprocess :: MaplePoly -> MaplePoly -> MapleVar -> MapleVar 
           -> (MapleCoeffs, MapleCoeffs, MapleInt, MapleInt)
preprocess p q u v =
  let
    r  = mapleEvalN "preprocess" [p, q, u, v]
    a = r!!0
    b = r!!1
    cb = r!!2
    db = r!!3
  in (a, b, cb, db)

------------------------------------------------------------------------------
-- list of modules to use for polynomials a and b
------------------------------------------------------------------------------
modlist :: MapleCoeffs -> MapleCoeffs -> MapleInt -> MapleInt 
        -> ([MapleInt], MapleInt)
modlist a b cb db = 
  let
    ms = mapleEvalN "modlist" [a, b, cb, db]
  in
    (tail ms, head ms)

------------------------------------------------------------------------------
-- modular resultant 
------------------------------------------------------------------------------
mresultant :: MapleInt -> MapleVar -> MapleCoeffs -> MapleCoeffs -> MapleInt
           -> [MapleInt]
mresultant m v a b db = mapleEvalN "mresultant" [m, v, a, b, db]

------------------------------------------------------------------------------
-- chinese remaindering algorithm
------------------------------------------------------------------------------
cra :: [ MapleInt ] -> [ MapleInt ] -> MapleInt
cra cs ms = mapleEval "cra" [mlist cs, mlist ms]

------------------------------------------------------------------------------
-- transform Maple coefficient list to Polynomial
------------------------------------------------------------------------------
polynomial :: [ MapleInt ] -> MapleVar -> MapleInt -> MaplePoly
polynomial c v p = mapleEval "polynomial" [mlist c, v, p]

------------------------------------------------------------------------------
-- transform Haskell list to Maple list
------------------------------------------------------------------------------
mlist :: [ MapleInt ] -> MapleCoeffs
mlist x = mapleEval "mlist" x

------------------------------------------------------------------------------
--
-- main program
--
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- compute string representation of resultant of polynomials 
-- represented by string arguments
------------------------------------------------------------------------------
sresultant :: String -> String -> String -> String -> String
sresultant ps qs us vs =
  let
    p = string2MapleExpr ps
    q = string2MapleExpr qs
    u = string2MapleExpr us
    v = string2MapleExpr vs
    r = resultant p q u v
  in
    mapleObject2String r
    
------------------------------------------------------------------------------
-- read (p(u,v), q(u,v), u, v) as separate strings from stdin
-- and print resultant w.r.t. u to stdout
------------------------------------------------------------------------------
main :: IO ()
main = 
  do
#if !defined(__PARALLEL_HASKELL__)
    mapleInit 
#endif
    ps <- getLine 
    qs <- getLine 
    us <- getLine 
    vs <- getLine 
    t1 <- getClockTime
    putStrLn (show t1)
    putStrLn (sresultant ps qs us vs)
    t2 <- getClockTime
    putStrLn (show (diffClockTimes t2 t1))
#if !defined(__PARALLEL_HASKELL__)
    mapleTerm
#endif

------------------------------------------------------------------------------
-- $Id: resultant.hs,v 1.1.1.1 2004/06/21 07:38:26 rmartine Exp $
------------------------------------------------------------------------------
